#include "aloha.h"

/*! Charge les plugins prsents dans la liste des rpertoires contenue
 * dans le fichier ~/.aloha.cfg
 * \return 1 si le chargement russi.
 */
int lire_plugins_home(void)
{
    FILE * alohacfg = NULL;
    char * tmp = NULL, * home = NULL;
    int i, len;

    home = getenv("HOME");

    if ( home != NULL ) {
	i = strlen(home);
	tmp = malloc(i+12); // 12 = strlen("/.aloha.cfg")+1
	if ( tmp != NULL ) {
	    strcpy(tmp, home);
	    strcat(tmp, "/.aloha.cfg");

	    alohacfg = fopen(tmp, "r");
	    free(tmp);
	    if ( alohacfg != NULL ) {
		printf("Chargement de ~/.aloha.cfg\n");
		tmp = NULL; i = 0;
		while ( (len = getline(&tmp, &i, alohacfg)) != -1 ) {
		    tmp[len-1] = 0;
		    printf("%s\n",tmp);
		    if ( ! charge_rep(tmp) )
			return 0;
		}
		free(tmp);
	    }
	    fclose(alohacfg);
	} else {
	    perror("malloc");
	}
    }

    return 1;
}

/*! Charge les plugins prsents dans la liste des rpertoires contenue
 * dans la variable d'environnement ALOHA_PATH
 * \return 1 si le chargement russi.
 */
int lire_plugins_env(void)
{
    char * ajout = NULL, * suiv = NULL;
    
    ajout = getenv("ALOHA_PATH");

    if ( ajout != NULL ) {
	suiv = ajout;
	while ( *ajout != 0 ) {
	    // On cherche la prochaine entre
	    while ( *suiv != 0 && *suiv != ':' )
		suiv++;
	    if ( *suiv == ':' ) {
		*suiv = 0;
		suiv++;
	    }
	    // On charge le rpertoire
	    if ( ! charge_rep(ajout) )
		return 0;
	    ajout = suiv;
	}
    }

    return 1;
}

/*! Charge les plugins
 */
int lire_plugins(void)
{
    if ( ! lire_plugins_home() )
	return 0;

    if ( ! lire_plugins_env() )
	return 0;

    return 1;
}

/*! En fonction des espaces, "transforme" une chaine de caractre en tableau
 * de chaines contenant les mots de la premire chaine. Exemple :
 * "hello world" -> ["hello","world"]
 * \param l chaine de caractre  "parser"
 * \param argc pointeur sur un entier qui contiendra le nombre d'arguments
 * \param pointeur sur un tableau de chaine de caractres qui contiendra les arguments
 * \return 1 si le "parsing" russi.
 */
int parse(char * ligne, int * argc, char *** argv)
{
    int i=0,j=0;

    *argc = 0;

    for ( i=0, j=0 ; ligne[i] != 0 ; i++ )
	if ( ligne[i] == 0 || ligne[i] == ' ' ) {
	    ligne[i] = 0;
	    *argv = realloc(*argv, sizeof(char **) * (*argc+1));
	    if ( *argv == NULL ) {
		perror("realloc");
		return 0;
	    }
	    (*argv)[*argc] = ligne+j;
	    j = i+1;
	    (*argc)++;
	}
    
    if ( ligne[i] == 0 && argc != 0 ) {
	ligne[i] = 0;
	*argv = realloc(*argv, sizeof(char **) * (*argc+1));
	if ( *argv == NULL ) {
	    perror("realloc");
	    return 0;
	}
	(*argv)[*argc] = ligne+j;
	j = i+1;
	(*argc)++;
    }
    
    return 1;
}

int main(void)
{
    pile p = NULL;
    char * ligne = NULL;
    int argc = 0;
    char ** argv = NULL;

    init_plugins();

    if ( ! lire_plugins() ) {
	fprintf(stderr,"Erreur fatale  l'initialisation des plugins\n");
	return 1;
    }

    tri_fonctions();

    while ( (ligne = readline("\e[1;31maloha > \e[0m")) ) {
	if ( ligne && *ligne ) {
	    add_history(ligne);

	    parse(ligne,&argc,&argv);
	    execute(&p,argc,argv);
	    
	    free(ligne);
	    free(argv);
	    argv = NULL;
	}
    }

    effacer_fonctions();
    detruirepile(p);
    
    exit(EXIT_SUCCESS);
}
