#include "plugins.h"

//! Variable globale statique contenant toutes les fonctions du programme
static fonctions * f = NULL;

/*! initialise le systme de plugins
 * \return 1 si l'initialisation est russie, 0 sinon
 */
int init_plugins(void)
{
    f = calloc(1,sizeof(fonctions));
    if ( f == NULL ) {
	perror("calloc");
	return 0;
    }

    return 1;
}

/*! Ajoute la fonction "nom" du fichier "fichier"  la liste des fonctions
 * \param nom nom de la fonction  ajouter
 * \param fichier nom du fichier contenant la fonction
 * \return 1 si l'initialisation est russie, 0 sinon
 */
int ajouter_fonction(char * nom, char * fichier)
{
    assert(f != NULL);

    if ( f->nbf == f->taille_buf ) {
	f->f = realloc(f->f, sizeof(fonction)*(f->taille_buf+42));
	if ( f->f == NULL ) {
	    perror("realloc");
	    return 0;
	}
	f->taille_buf += 42;
    }

    f->f[f->nbf].nom = malloc(sizeof(char)*(strlen(nom)+1));
    if ( f->f[f->nbf].nom == NULL ) {
	perror("malloc");
	return 0;
    }
    strcpy(f->f[f->nbf].nom, nom);

    f->f[f->nbf].fichier = malloc(sizeof(char)*(strlen(fichier)+1));
    if ( f->f[f->nbf].fichier == NULL ) {
	perror("malloc");
	free(f->f[f->nbf].nom);
	return 0;
    }
    strcpy(f->f[f->nbf].fichier,fichier);

    f->f[f->nbf].handle = NULL;

    f->f[f->nbf].f = charge_fonction;

    f->nbf++;

    return 1;
}

/*! Ajoute toutes les fonctions de la bibliothque "nomfichier"  la liste
 * des fonctions
 * \param nomfichier nom du fichier  charger
 * \return 1 si l'initialisation est russie, 0 sinon
 */
int charge_fichier(char * nomfichier)
{
    assert(nomfichier != NULL);

    char ** (* getPlugins)(void);
    void * handle;
    char ** liste = NULL, ** debut_liste = NULL, * error = NULL;
    
    printf("Chargement de %s... ",basename(nomfichier));

    dlerror();
    handle = dlopen(nomfichier, RTLD_LAZY);
    if ( handle == NULL ) {
	fprintf(stderr,"%s\n",dlerror());
	return 0;
    }

    getPlugins = dlsym(handle, "getPlugins");
    if ( (error = dlerror()) != NULL ) {
	fprintf(stderr,"%s\n",error);
	return 0;
    }

    liste = getPlugins();
    if ( liste != NULL ) {
	printf("[ ");
	debut_liste = liste;
	while ( *liste != NULL ) {
	    if ( ! ajouter_fonction(*liste, nomfichier) )
		return 0;
	    printf("%s ",*liste);
	    free(*liste);
	    liste++;
	}
	free(debut_liste);
	printf("]");
    }

    putchar('\n');
    
    dlclose(handle);

    return 1;
}

/*! Charge toutes les bibliothques du rpertoire donn
 * \param nom_rep rpertoire  charger
 * \return 1 si l'initialisation est russie, 0 sinon
 */
int charge_rep(char * nom_rep)
{
    assert(nom_rep != NULL);

    DIR * dir;
    struct dirent * dirent;
    char lib[MAXPATHLEN];

    dir = opendir(nom_rep);
    if ( dir == NULL ) {
	fprintf(stderr,"%s ",nom_rep);
	perror("opendir");
	return 0;
    }

    while ( (dirent = readdir(dir)) != NULL )
	if ( strcmp(dirent->d_name, ".") != 0
		&& strcmp(dirent->d_name, "..") != 0 ) {
	    strcpy(lib, nom_rep);
	    strcat(lib, dirent->d_name);
	    if ( ! charge_fichier(lib) )
		return 0;
	}

    if ( closedir(dir) == -1 )
	perror("closedir");

    return 1;
}

/*! Fonction de prchargement
 * Au premier appel d'une fonction de la liste des fonctions, cette fonction
 * charge la fonction appelle dfinitivement
 * \param p pointeur sur la pile de liste de chaines
 * \param argc nombre d'arguments de la commande  xcuter
 * \param argv tableau de chaines contenant les arguments  xcuter
 * \return -1 si l'appel de la fonction a chou, autre chose sinon
 */
int charge_fonction(pile * p, int argc, char ** argv)
{
    if ( argc < 1 )
	return -1;

    int pos_fonction;

    pos_fonction = recherche_fonction(argv[0]);
    if ( pos_fonction == -1 )
	return -1;

    dlerror();
    f->f[pos_fonction].handle = dlopen(f->f[pos_fonction].fichier, RTLD_LAZY);
    if ( f->f[pos_fonction].handle == NULL ) {
	fprintf(stderr,"%s\n",dlerror());
	return -1;
    }

    f->f[pos_fonction].f = 
	dlsym(f->f[pos_fonction].handle, f->f[pos_fonction].nom);

    if ( f->f[pos_fonction].f == NULL )
	return -1;

    return f->f[pos_fonction].f(p,argc,argv);
}

/*! Retourne le numro de la fonction "nom" ou -1 si la fonction n'existe pas
 * ( recherche dichotomique )
 * ATTENTION : La liste des fonctions doit avoir t trie !
 * \param nom fonction  rechercher
 * \return -1 si la fonction est introuvable, son numro sinon
 */
int recherche_fonction(char * nom)
{
    int p, res, debut = 0, fin = f->nbf - 1;

    while ( debut != fin ) {
	p = (debut+fin)/2;
	if ( (res = strcmp(nom, f->f[p].nom)) == 0 )
	    return p;
	if ( res < 0 )
	    fin = p;
	else
	    debut = p+1;
    }

    if ( strcmp(nom, f->f[debut].nom) == 0 )
	return debut;

    return -1;
}

/*! execute la fonction argv[0] aprs l'avoir trouve dans la liste des fonctions.
 * \param p pointeur sur la pile de liste de chaines
 * \param argc nombre d'arguments de la commande  xcuter
 * \param argv tableau de chaines contenant les arguments  xcuter
 * \return -1 si la fonction n'existe pas, autre chose sinon.
 */
int execute(pile * p, int argc, char ** argv)
{
    if ( argc < 1 )
	return -1;

    int pos_fonction = recherche_fonction(argv[0]);
    if ( pos_fonction == -1 )
	return -1;

    return f->f[pos_fonction].f(p,argc,argv);
}

/*! Efface toutes les fonctions enregistres
 */
void effacer_fonctions(void)
{
    int i;
    for ( i=0 ; i < f->nbf ; i++ ) {
	if ( f->f[i].handle != NULL )
	    dlclose(f->f[i].handle);
	free(f->f[i].nom);
	free(f->f[i].fichier);
    }
    free(f->f);
    free(f);
}

/*! Fonction utilise pour le tri
 * \param a fonction  comparer
 * \param b fonction  comparer
 * \return 0 si les des fonctions sont identiques, une valeur ngative si la 
 * premire a un nom plus petit que celle de la seconde, positive si elle est
 * suprieure.
 */
int compar(const void * a, const void * b)
{
    return strcmp(((fonction *)a)->nom, ((fonction *)b)->nom);
}

/*! Trie toutes les fonctions par ordre alphabtique
 */
void tri_fonctions(void)
{
    qsort(f->f, f->nbf, sizeof(fonction), compar);
}
